#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <signal.h>

#include "data.h"

extern PrintHead();
extern PrintTail();
static void terminate();

int main( argc, argv )
int argc;
char *argv[];
{
	int fd, temp;
	int i, c;
	int n;

	if( argc < 2 ) {
		fprintf( stderr, "Help for htlpr :  %s -h\n", argv[0] );
		return 1;
	}

	signal( SIGINT, terminate );
	signal( SIGKILL, terminate );
	signal( SIGQUIT, terminate );

	n = 0;
	for( i = 1; i < argc; i++ ) {
		if( argv[i][0] == '-' ) {
			if( setoption( argv[i] ) < 0 ) {
				fprintf( stderr, "Unknown option %s\n", argv[i] );
				return 1;
			}
		}
		else 
			n++;
	}
	if( n == 0 )
		return;
	if( ( temp = opentemp() ) < 0 ) {
		fprintf( stderr, "Cannot open tempfile\n" );
		return 1;
	}
	else {
		PrintHead( temp );
	}

	for( i = 1; i < argc; i++ ) {
		if( argv[i][0] == '-' )            /* If option, skip */
			continue;

		if( ( fd = open( argv[i], O_RDONLY ) ) < 0 ) {
			fprintf( stderr, "No Such a file %s.\n", argv[i] );
			return 1;
		}

		printf( "Converting %s to Latex Document ", argv[i] );
		fflush( stdout );
		if( ConvertToLatex( fd, temp ) < 0 ) {
			fprintf( stderr, "Unknown error.\n" );
			return 1;
		}
		close( fd );
		printf( "... Done\n" );
	}
	PrintTail( temp );
	close( temp );

	getcwd( curdir, FILELENGTH );
	chdir( TMPDIR );
	printf( "Running %s. Waiting ", LATEX );
	fflush( stdout );
	if( DoExecuteLatex() < 0 )
		return 1; 
	printf( " Done\n" );

	if( DoExecuteDvips() < 0 ) 
		return 1;

	if( output == PRN ) {
		if( DoPrint() < 0 ) 
			return 1;
	}
	else if( output == PS ) {
		if( DoPs() < 0 ) 
			return 1;
	}
	ClearTempFile();
}

static void terminate()
{
	ClearTempFile();
	exit( 1 );
}
