#include <stdio.h>

#include "data.h"

void printset( char [] );
void margin( char [] );
void setfonts( char [] );
void sethfonts( char [] );
void setpapersize( char [] );
int outputf( char [] );
void setfontsize( char [] );
void settabsize( char [] );
void settitle( char [] );
void showhelp( void );

int setoption( char op[] )
{
	int val;

	switch( op[1] ) {
		case 'f':
			setfonts( op );
			break;
		case 'k':
			sethfonts( op );
			break;
		case 'd':
			setpapersize( op );
			break;
		case 'P':
			printset( op );
			break;
		case 'm':
			margin( op );
			break;
		case 'o':
			if( outputf( op ) < 0 ) {
				fprintf( stderr, "[Usage] htlpr -ooutfile text\n" );
				exit( 1 );
			};
			output = PS;
			break;
		case 's':
			setfontsize( op );
			break;
		case 't':
			settabsize( op );
			break;
		case 'p':
			pagenum = YES;
			break;
		case 'b':
			border = YES;
			break;
		case 'h':
			showhelp();
			exit( 0 );
		case 'H':
			settitle( op );
			break;
		default:
			fprintf( stderr, "Unknown option \"%c\" ( ignored )\n", op[1] );
			break;
	} 
	return 0;
}

void printset( char op[] )
{
	if( op[2] )
		strcpy( printer, (op+2) );
}

void margin( char op[] )
{
	int i, j;
	char opt[20];

	i = 2;
	j = 0;
	while( op[i] && op[i] != ',' ) {
		opt[j] = op[i];
		i++;
		j++;
	}
	opt[j] = '\0';
	if( opt[0] )
		sprintf( marginx, opt );
	j = 0;
	if( op[i++] == ',' ) {
		while( op[i] ) {
			opt[j] = op[i];
			i++;
			j++;
		}
	}
	else
		return;
	opt[j] = '\0';
	if( opt[0] )
		sprintf( marginy, opt );
}

int outputf( char op[] )
{
	if( !op[2] )
		return -1;
	sprintf( outputfile, (op+2) );
	return 0;
}

void setfontsize( char op[] )
{
	char fs[30];
	int i;

	if( !op[2] )
		exit( 0 );
	sprintf( fs, op+2 );
	for( i = 0; fontsize[i]; i++ ) {
		if( strcmp( fontsize[i], fs ) == 0 ) {
			fsize = i;
			return;
		}
	}
	printf( "[Usage] htlpr -sfontsize text\n" );
	printf( "fontsize : " );
	for( i = 0; fontsize[i]; i++ ) 
		printf( "%s\n", fontsize[i] );
	printf( "\n" );
	exit( 1 );
}

void setpapersize( char op[] )
{
	char fs[30];
	int i;

	if( !op[2] )
		exit( 1 );
	sprintf( fs, op+2 );
	for( i = 0; papersize[i]; i++ ) {
		if( strcmp( papersize[i], fs ) == 0 ) {
			psize = i;
			return;
		}
	}
	printf( "[Usage] htlpr -dpaper-size text\n" );
	printf( "papersize : " );
	for( i = 0; papersize[i]; i++ ) 
		printf( "%s\n", papersize[i] );
	printf( "\n" );
	exit( 1 );
}

void settitle( char op[] )
{
	if( !op[2] ) 
		return;
	sprintf( Title, op+2 );
	title = YES;
}	

void settabsize( char op[] )
{
	char ts[10];

	if( !op[2] )
		return;
	sprintf( ts, op+2 );
	if( ( tab = atoi( ts ) ) <= 0 )
		tab = TAB;
}	

void setfonts( char op[] )
{
	char fs[30];
	int i;

	if( !op[2] )
		;
	else {
		sprintf( fs, op+2 );
		for( i = 0; fontstyle[i]; i++ ) {
			if( strcmp( fontstyle[i], fs ) == 0 ) {
				fstyle = i;
				return;
			}
		}
	}
	printf( "[Usage] htlpr -ffontstyle text\n" );
	printf( "fontstyle : \n" );
	for( i = 0; fontstyle[i]; i++ ) 
		printf( "%s\t", fontstyle[i] );
	printf( "\n" );
	exit( 1 );
}

void sethfonts( char op[] )
{
	char fs[30];
	int i;

	if( !op[2] )
		;
	else {
		sprintf( fs, op+2 );
		for( i = 0; hfontstyle[i]; i++ ) {
			if( strcmp( hfontstyle[i], fs ) == 0 ) {
				hfstyle = i;
				return;
			}
		}
	}
	printf( "[Usage] htlpr -kfontstyle text\n" );
	printf( "hfontstyle : \n" );
	for( i = 0; hfontstyle[i]; i++ ) 
		printf( "%s\t", hfontstyle[i] );
	printf( "\n" );
	exit( 1 );
}

void showhelp( void )
{
	int i;
	int c;

	char *help[] = {
		"                      This is help for htlpr version 0.60", 
		"",
		"$ htlpr -option hangul_text",
		"",
		"[option]",
		"",
		"1) -ffonts",
		"   rm : Roman       it : Italic           sc : Small Caps",
		"   bf : Bold Face   sl : slanted          sf : Sans Seref",
		"   tt : Typewriter",
		"2) -khangul_fonts",
		"   mj : myoungjo        mjb : myoungjo-bold   gt : gothic hangul",
		"   gr : graphic hangul  dr  : dinaru-hangul   gs : butguelssi-hangul",
		"3) -ooutput.ps",
		"   Instead of printing, output output.ps", 
		"4) -Pprinter",
		"   printer : printer name",
		"5) -m-0.3,-0.4",	
		"   topmargin : -0.3, leftmargin : -0.4",
		"6) -sfontsize",
		"   tiny                 scriptsize      footnotesize           small", 
		"   normalsize(default)  large           Large                  LARGE",
		"   huge                 Huge",
		"7) -p",
		"   print page number",
		"8) -b",
		"   print border",
		"9) -t4",
		"   tabsize is 4( default 8 )",
		"10) -HTitle",
		"   Title ( default is none )",
		"11) -dpapersize",
		"   a4       letter",
		"   ( default is a4 )",
		"12) -h",
		"   This help ",
		( char * )0
	};		

	i = 0;
	while( help[i] ) {
		printf( "%s\n", help[i] );
		i++;
		if( !( i % 23 ) ) {
			printf( "--[7m Press Return [0m--" );
			if( ( c = getchar() ) == 'q' || c == 'Q' )
				return;
		}
	}
}	
